from os import path

from typing_extensions import Self
from pydantic import BaseModel, Field, model_validator

class Config(BaseModel):
    api: dict[str, dict[str, set[str]]]
    resolvers: list[str]
    log_path: str | None = Field(alias = 'log-path', default = None)

    @model_validator(mode = 'after')
    def check_if_api_key_is_path(self) -> Self:
        for key in set(self.api.keys()):
            if path.exists(key):
                with open(key, encoding = 'utf-8') as file:
                    api_key = file.read().strip()
                    self.api[api_key] = self.api.pop(key)

        return self
